@@ -1,12 +1,22 @@
 /****************************************************************************
 **
 ** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the examples of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:BSD$
-** You may use this file under the terms of the BSD license as follows:
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
+**
+** BSD License Usage
+** Alternatively, you may use this file under the terms of the BSD license
+** as follows:
 **
 ** "Redistribution and use in source and binary forms, with or without
 ** modification, are permitted provided that the following conditions are
@@ -40,82 +50,101 @@
 
 #include "browser.h"
 #include "browserwindow.h"
+#include "downloadmanagerwidget.h"
 #include "tabwidget.h"
-#include "urllineedit.h"
 #include "webview.h"
 #include <QApplication>
 #include <QCloseEvent>
 #include <QDesktopWidget>
+#include <QEvent>
 #include <QFileDialog>
+#include <QInputDialog>
 #include <QMenuBar>
 #include <QMessageBox>
 #include <QProgressBar>
+#include <QScreen>
 #include <QStatusBar>
 #include <QToolBar>
 #include <QVBoxLayout>
+#include <QWebEngineProfile>
 
-BrowserWindow::BrowserWindow(QWidget *parent, Qt::WindowFlags flags)
-    : QMainWindow(parent, flags)
-    , m_tabWidget(new TabWidget(this))
-    , m_progressBar(new QProgressBar(this))
+BrowserWindow::BrowserWindow(Browser *browser, QWebEngineProfile *profile, bool forDevTools)
+    : m_browser(browser)
+    , m_profile(profile)
+    , m_tabWidget(new TabWidget(profile, this))
+    , m_progressBar(nullptr)
     , m_historyBackAction(nullptr)
     , m_historyForwardAction(nullptr)
     , m_stopAction(nullptr)
     , m_reloadAction(nullptr)
     , m_stopReloadAction(nullptr)
-    , m_urlLineEdit(new UrlLineEdit(this))
+    , m_urlLineEdit(nullptr)
+    , m_favAction(nullptr)
 {
-    setToolButtonStyle(Qt::ToolButtonFollowStyle);
     setAttribute(Qt::WA_DeleteOnClose, true);
+    setFocusPolicy(Qt::ClickFocus);
 
-    QToolBar *toolbar = createToolBar();
-    addToolBar(toolbar);
-    menuBar()->addMenu(createFileMenu(m_tabWidget));
-    menuBar()->addMenu(createViewMenu(toolbar));
-    menuBar()->addMenu(createWindowMenu(m_tabWidget));
-    menuBar()->addMenu(createHelpMenu());
+    if (!forDevTools) {
+        m_progressBar = new QProgressBar(this);
+
+        QToolBar *toolbar = createToolBar();
+        addToolBar(toolbar);
+        menuBar()->addMenu(createFileMenu(m_tabWidget));
+        menuBar()->addMenu(createEditMenu());
+        menuBar()->addMenu(createViewMenu(toolbar));
+        menuBar()->addMenu(createWindowMenu(m_tabWidget));
+        menuBar()->addMenu(createHelpMenu());
+    }
 
     QWidget *centralWidget = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout;
     layout->setSpacing(0);
     layout->setMargin(0);
-    addToolBarBreak();
+    if (!forDevTools) {
+        addToolBarBreak();
 
-    m_progressBar->setMaximumHeight(1);
-    m_progressBar->setTextVisible(false);
-    m_progressBar->setStyleSheet(QStringLiteral("QProgressBar {border: 0px } QProgressBar::chunk { background-color: red; }"));
+        m_progressBar->setMaximumHeight(1);
+        m_progressBar->setTextVisible(false);
+        m_progressBar->setStyleSheet(QStringLiteral("QProgressBar {border: 0px} QProgressBar::chunk {background-color: #da4453}"));
+
+        layout->addWidget(m_progressBar);
+    }
 
-    layout->addWidget(m_progressBar);
     layout->addWidget(m_tabWidget);
     centralWidget->setLayout(layout);
     setCentralWidget(centralWidget);
 
     connect(m_tabWidget, &TabWidget::titleChanged, this, &BrowserWindow::handleWebViewTitleChanged);
-    connect(m_tabWidget, &TabWidget::linkHovered, [this](const QString& url) {
-        statusBar()->showMessage(url);
-    });
-    connect(m_tabWidget, &TabWidget::loadProgress, this, &BrowserWindow::handleWebViewLoadProgress);
-    connect(m_tabWidget, &TabWidget::urlChanged, this, &BrowserWindow::handleWebViewUrlChanged);
-    connect(m_tabWidget, &TabWidget::iconChanged, this, &BrowserWindow::handleWebViewIconChanged);
-    connect(m_tabWidget, &TabWidget::webActionEnabledChanged, this, &BrowserWindow::handleWebActionEnabledChanged);
-    connect(m_urlLineEdit, &QLineEdit::returnPressed, this, [this]() {
-        m_urlLineEdit->setFavIcon(QIcon(QStringLiteral(":defaulticon.png")));
-        loadPage(m_urlLineEdit->url());
-    });
+    if (!forDevTools) {
+        connect(m_tabWidget, &TabWidget::linkHovered, [this](const QString& url) {
+            statusBar()->showMessage(url);
+        });
+        connect(m_tabWidget, &TabWidget::loadProgress, this, &BrowserWindow::handleWebViewLoadProgress);
+        connect(m_tabWidget, &TabWidget::webActionEnabledChanged, this, &BrowserWindow::handleWebActionEnabledChanged);
+        connect(m_tabWidget, &TabWidget::urlChanged, [this](const QUrl &url) {
+            m_urlLineEdit->setText(url.toDisplayString());
+        });
+        connect(m_tabWidget, &TabWidget::favIconChanged, m_favAction, &QAction::setIcon);
+        connect(m_tabWidget, &TabWidget::devToolsRequested, this, &BrowserWindow::handleDevToolsRequested);
+        connect(m_urlLineEdit, &QLineEdit::returnPressed, [this]() {
+            m_tabWidget->setUrl(QUrl::fromUserInput(m_urlLineEdit->text()));
+        });
 
-    m_urlLineEdit->setFavIcon(QIcon(QStringLiteral(":defaulticon.png")));
+        QAction *focusUrlLineEditAction = new QAction(this);
+        addAction(focusUrlLineEditAction);
+        focusUrlLineEditAction->setShortcut(QKeySequence(Qt::CTRL | Qt::Key_L));
+        connect(focusUrlLineEditAction, &QAction::triggered, this, [this] () {
+            m_urlLineEdit->setFocus(Qt::ShortcutFocusReason);
+        });
+    }
 
-    handleWebViewTitleChanged(tr("Qt Simple Browser"));
+    handleWebViewTitleChanged(QString());
     m_tabWidget->createTab();
 }
 
-BrowserWindow::~BrowserWindow()
-{
-}
-
 QSize BrowserWindow::sizeHint() const
 {
-    QRect desktopRect = QApplication::desktop()->screenGeometry();
+    QRect desktopRect = QApplication::primaryScreen()->geometry();
     QSize size = desktopRect.size() * qreal(0.9);
     return size;
 }
@@ -124,19 +153,21 @@ QMenu *BrowserWindow::createFileMenu(TabWidget *tabWidget)
 {
     QMenu *fileMenu = new QMenu(tr("&File"));
     fileMenu->addAction(tr("&New Window"), this, &BrowserWindow::handleNewWindowTriggered, QKeySequence::New);
+    fileMenu->addAction(tr("New &Incognito Window"), this, &BrowserWindow::handleNewIncognitoWindowTriggered);
 
-    QAction *newTabAction = new QAction(QIcon(QLatin1String(":addtab.png")), tr("New &Tab"), this);
+    QAction *newTabAction = new QAction(tr("New &Tab"), this);
     newTabAction->setShortcuts(QKeySequence::AddTab);
-    newTabAction->setIconVisibleInMenu(false);
-    connect(newTabAction, &QAction::triggered, tabWidget, &TabWidget::createTab);
+    connect(newTabAction, &QAction::triggered, this, [this]() {
+        m_tabWidget->createTab();
+        m_urlLineEdit->setFocus();
+    });
     fileMenu->addAction(newTabAction);
 
     fileMenu->addAction(tr("&Open File..."), this, &BrowserWindow::handleFileOpenTriggered, QKeySequence::Open);
     fileMenu->addSeparator();
 
-    QAction *closeTabAction = new QAction(QIcon(QLatin1String(":closetab.png")), tr("&Close Tab"), this);
+    QAction *closeTabAction = new QAction(tr("&Close Tab"), this);
     closeTabAction->setShortcuts(QKeySequence::Close);
-    closeTabAction->setIconVisibleInMenu(false);
     connect(closeTabAction, &QAction::triggered, [tabWidget]() {
         tabWidget->closeTab(tabWidget->currentIndex());
     });
@@ -147,8 +178,8 @@ QMenu *BrowserWindow::createFileMenu(TabWidget *tabWidget)
     connect(closeAction, &QAction::triggered, this, &QWidget::close);
     fileMenu->addAction(closeAction);
 
-    connect(fileMenu, &QMenu::aboutToShow, [closeAction]() {
-        if (Browser::instance().windows().count() == 1)
+    connect(fileMenu, &QMenu::aboutToShow, [this, closeAction]() {
+        if (m_browser->windows().count() == 1)
             closeAction->setText(tr("&Quit"));
         else
             closeAction->setText(tr("&Close Window"));
@@ -156,6 +187,32 @@ QMenu *BrowserWindow::createFileMenu(TabWidget *tabWidget)
     return fileMenu;
 }
 
+QMenu *BrowserWindow::createEditMenu()
+{
+    QMenu *editMenu = new QMenu(tr("&Edit"));
+    QAction *findAction = editMenu->addAction(tr("&Find"));
+    findAction->setShortcuts(QKeySequence::Find);
+    connect(findAction, &QAction::triggered, this, &BrowserWindow::handleFindActionTriggered);
+
+    QAction *findNextAction = editMenu->addAction(tr("Find &Next"));
+    findNextAction->setShortcut(QKeySequence::FindNext);
+    connect(findNextAction, &QAction::triggered, [this]() {
+        if (!currentTab() || m_lastSearch.isEmpty())
+            return;
+        currentTab()->findText(m_lastSearch);
+    });
+
+    QAction *findPreviousAction = editMenu->addAction(tr("Find &Previous"));
+    findPreviousAction->setShortcut(QKeySequence::FindPrevious);
+    connect(findPreviousAction, &QAction::triggered, [this]() {
+        if (!currentTab() || m_lastSearch.isEmpty())
+            return;
+        currentTab()->findText(m_lastSearch, QWebEnginePage::FindBackward);
+    });
+
+    return editMenu;
+}
+
 QMenu *BrowserWindow::createViewMenu(QToolBar *toolbar)
 {
     QMenu *viewMenu = new QMenu(tr("&View"));
@@ -253,7 +310,7 @@ QMenu *BrowserWindow::createWindowMenu(TabWidget *tabWidget)
         menu->addAction(previousTabAction);
         menu->addSeparator();
 
-        QVector<BrowserWindow*> windows = Browser::instance().windows();
+        QVector<BrowserWindow*> windows = m_browser->windows();
         int index(-1);
         for (auto window : windows) {
             QAction *action = menu->addAction(window->windowTitle(), this, &BrowserWindow::handleShowWindowTriggered);
@@ -276,7 +333,7 @@ QMenu *BrowserWindow::createHelpMenu()
 QToolBar *BrowserWindow::createToolBar()
 {
     QToolBar *navigationBar = new QToolBar(tr("Navigation"));
-    navigationBar->setAllowedAreas(Qt::TopToolBarArea | Qt::BottomToolBarArea);
+    navigationBar->setMovable(false);
     navigationBar->toggleViewAction()->setEnabled(false);
 
     m_historyBackAction = new QAction(this);
@@ -293,6 +350,7 @@ QToolBar *BrowserWindow::createToolBar()
     m_historyBackAction->setShortcuts(backShortcuts);
     m_historyBackAction->setIconVisibleInMenu(false);
     m_historyBackAction->setIcon(QIcon(QStringLiteral(":go-previous.png")));
+    m_historyBackAction->setToolTip(tr("Go back in history"));
     connect(m_historyBackAction, &QAction::triggered, [this]() {
         m_tabWidget->triggerWebPageAction(QWebEnginePage::Back);
     });
@@ -310,6 +368,7 @@ QToolBar *BrowserWindow::createToolBar()
     m_historyForwardAction->setShortcuts(fwdShortcuts);
     m_historyForwardAction->setIconVisibleInMenu(false);
     m_historyForwardAction->setIcon(QIcon(QStringLiteral(":go-next.png")));
+    m_historyForwardAction->setToolTip(tr("Go forward in history"));
     connect(m_historyForwardAction, &QAction::triggered, [this]() {
         m_tabWidget->triggerWebPageAction(QWebEnginePage::Forward);
     });
@@ -320,24 +379,24 @@ QToolBar *BrowserWindow::createToolBar()
         m_tabWidget->triggerWebPageAction(QWebEnginePage::WebAction(m_stopReloadAction->data().toInt()));
     });
     navigationBar->addAction(m_stopReloadAction);
+
+    m_urlLineEdit = new QLineEdit(this);
+    m_favAction = new QAction(this);
+    m_urlLineEdit->addAction(m_favAction, QLineEdit::LeadingPosition);
+    m_urlLineEdit->setClearButtonEnabled(true);
     navigationBar->addWidget(m_urlLineEdit);
-    int size = m_urlLineEdit->sizeHint().height();
-    navigationBar->setIconSize(QSize(size, size));
+
+    auto downloadsAction = new QAction(this);
+    downloadsAction->setIcon(QIcon(QStringLiteral(":go-bottom.png")));
+    downloadsAction->setToolTip(tr("Show downloads"));
+    navigationBar->addAction(downloadsAction);
+    connect(downloadsAction, &QAction::triggered, [this]() {
+        m_browser->downloadManagerWidget().show();
+    });
+
     return navigationBar;
 }
 
-void BrowserWindow::handleWebViewIconChanged(const QIcon &icon)
-{
-    m_urlLineEdit->setFavIcon(icon);
-}
-
-void BrowserWindow::handleWebViewUrlChanged(const QUrl &url)
-{
-    m_urlLineEdit->setUrl(url);
-    if (url.isEmpty())
-        m_urlLineEdit->setFocus();
-}
-
 void BrowserWindow::handleWebActionEnabledChanged(QWebEnginePage::WebAction action, bool enabled)
 {
     switch (action) {
@@ -354,32 +413,58 @@ void BrowserWindow::handleWebActionEnabledChanged(QWebEnginePage::WebAction acti
         m_stopAction->setEnabled(enabled);
         break;
     default:
-        qWarning("Unhandled webActionChanged singal");
+        qWarning("Unhandled webActionChanged signal");
     }
 }
 
 void BrowserWindow::handleWebViewTitleChanged(const QString &title)
 {
+    QString suffix = m_profile->isOffTheRecord()
+        ? tr("Qt Simple Browser (Incognito)")
+        : tr("Qt Simple Browser");
+
     if (title.isEmpty())
-        setWindowTitle(tr("Qt Simple Browser"));
+        setWindowTitle(suffix);
     else
-        setWindowTitle(tr("%1 - Qt Simple Browser").arg(title));
+        setWindowTitle(title + " - " + suffix);
 }
 
 void BrowserWindow::handleNewWindowTriggered()
 {
-    BrowserWindow *window = new BrowserWindow();
-    Browser::instance().addWindow(window);
-    window->loadHomePage();
+    BrowserWindow *window = m_browser->createWindow();
+    window->m_urlLineEdit->setFocus();
+}
+
+void BrowserWindow::handleNewIncognitoWindowTriggered()
+{
+    BrowserWindow *window = m_browser->createWindow(/* offTheRecord: */ true);
+    window->m_urlLineEdit->setFocus();
 }
 
 void BrowserWindow::handleFileOpenTriggered()
 {
-    QString file = QFileDialog::getOpenFileName(this, tr("Open Web Resource"), QString(),
+    QUrl url = QFileDialog::getOpenFileUrl(this, tr("Open Web Resource"), QString(),
                                                 tr("Web Resources (*.html *.htm *.svg *.png *.gif *.svgz);;All files (*.*)"));
-    if (file.isEmpty())
+    if (url.isEmpty())
         return;
-    loadPage(file);
+    currentTab()->setUrl(url);
+}
+
+void BrowserWindow::handleFindActionTriggered()
+{
+    if (!currentTab())
+        return;
+    bool ok = false;
+    QString search = QInputDialog::getText(this, tr("Find"),
+                                           tr("Find:"), QLineEdit::Normal,
+                                           m_lastSearch, &ok);
+    if (ok && !search.isEmpty()) {
+        m_lastSearch = search;
+        currentTab()->findText(m_lastSearch, 0, [this](bool found) {
+            if (!found)
+                statusBar()->showMessage(tr("\"%1\" not found.").arg(m_lastSearch));
+        });
+    }
 }
 
 void BrowserWindow::closeEvent(QCloseEvent *event)
@@ -398,25 +483,6 @@ void BrowserWindow::closeEvent(QCloseEvent *event)
     deleteLater();
 }
 
-
-void BrowserWindow::loadHomePage()
-{
-    loadPage(QStringLiteral("http://www.qt.io"));
-}
-
-void BrowserWindow::loadPage(const QString &page)
-{
-    loadPage(QUrl::fromUserInput(page));
-}
-
-void BrowserWindow::loadPage(const QUrl &url)
-{
-    if (url.isValid()) {
-        m_urlLineEdit->setUrl(url);
-        m_tabWidget->setUrl(url);
-    }
-}
-
 TabWidget *BrowserWindow::tabWidget() const
 {
     return m_tabWidget;
@@ -432,24 +498,31 @@ void BrowserWindow::handleWebViewLoadProgress(int progress)
     static QIcon stopIcon(QStringLiteral(":process-stop.png"));
     static QIcon reloadIcon(QStringLiteral(":view-refresh.png"));
 
-    if (progress < 100 && progress > 0) {
+    if (0 < progress && progress < 100) {
         m_stopReloadAction->setData(QWebEnginePage::Stop);
         m_stopReloadAction->setIcon(stopIcon);
         m_stopReloadAction->setToolTip(tr("Stop loading the current page"));
+        m_progressBar->setValue(progress);
     } else {
         m_stopReloadAction->setData(QWebEnginePage::Reload);
         m_stopReloadAction->setIcon(reloadIcon);
         m_stopReloadAction->setToolTip(tr("Reload the current page"));
+        m_progressBar->setValue(0);
     }
-    m_progressBar->setValue(progress < 100 ? progress : 0);
 }
 
 void BrowserWindow::handleShowWindowTriggered()
 {
     if (QAction *action = qobject_cast<QAction*>(sender())) {
         int offset = action->data().toInt();
-        QVector<BrowserWindow*> windows = Browser::instance().windows();
+        QVector<BrowserWindow*> windows = m_browser->windows();
         windows.at(offset)->activateWindow();
         windows.at(offset)->currentTab()->setFocus();
     }
 }
+
+void BrowserWindow::handleDevToolsRequested(QWebEnginePage *source)
+{
+    source->setDevToolsPage(m_browser->createDevToolsWindow()->currentTab()->page());
+    source->triggerAction(QWebEnginePage::InspectElement);
+}
